import os
import sys
import string
import firewire

# Bytes per frame
BPF = { 'PAL' : 144000, 'NTSC' : 120000 }

# Frames per second
FPS = { 'PAL' : 25, 'NTSC' : 30 }

# Configuration ROM Entries
CROM_VENDOR_ID  = 0x03
CROM_MODEL_ID   = 0x17
CROM_UNIT_SW_VERSION = 0x13
CSR_PROTAVC = 0x010001


def find_avc(fd, devs):
  """Return the description of the first AV/C device found
  or None if no AV/C device"""
  for dev in devs:
    if (dev['status'] == 1):
      crom = firewire.get_crom(fd, dev['eui'])
      for reg in crom['regs']:
        if ((reg['key'] == CROM_UNIT_SW_VERSION) and
           (reg['val'] == CSR_PROTAVC)):
            return dev
  return None


def get_text_leaf(fd, dev, key):
  """Return the text leaf information associated with the key from the
  device description"""
  next_is_text_leaf = None
  try:
    crom = firewire.get_crom(fd, dev['eui'])
    for reg in crom['regs']:
      if (reg['key'] == key):
        next_is_text_leaf = 1
      elif (next_is_text_leaf is not None):
        return reg['info']
    return ""
  except KeyError:
    return ""
  except TypeError:
    return ""


def file_exists(filename):
  try:
    os.stat(filename)
    return filename
  except OSError:
    return None


def get_vendor_from_oui(fd, dev):
  """Return the vendor ID string of a device from the OUI"""
  oui = -1
  try:
    crom = firewire.get_crom(fd, dev['eui'])
    for reg in crom['regs']:
      if (reg['key'] == CROM_VENDOR_ID):
        oui = reg['val']
        break
  except TypeError:
    pass
  oui = string.upper("%06x" % oui)
  try:
    if (file_exists("/usr/local/share/misc/OUI.db")):
      fd = open("/usr/local/share/misc/OUI.db", "r")
    elif (file_exists("OUI/OUI.db")):
      fd = open("OUI/OUI.db", "r")
    else:
      return ""
    line = fd.readline()
    while (line != ''):
      try:
        vendor_id, vendor = string.split(string.strip(line), "\t")
        if (vendor_id == oui):
          return vendor
      except ValueError:
        pass
      line = fd.readline()
    return "Unknown camcorder"  
  except IOError:
    return ""


def get_vendor(fd, dev):
  """Return the vendor ID string of a device"""
  vendor = get_text_leaf(fd, dev, CROM_VENDOR_ID)
  if (not vendor):
    return get_vendor_from_oui(fd, dev)
  return vendor


def get_model(fd, dev):
  """Return the model ID string of a device"""
  return get_text_leaf(fd, dev, CROM_MODEL_ID)


def get_eui64(fd, dev):
  """Return the formated EUI64 of a device"""
  crom = firewire.get_crom(fd, dev['eui'])
  return crom['EUI64'];


